@@ -1,16 +1,18 @@
 #' Process data from the DWD CDC FTP Server
 #' 
-#' Read climate variables (column meta data) from files downloaded with \code{\link{dataDWD}}.
-#' The metadata file is read, processed and returned as a data.frame.\cr
+#' Read climate variables (column meta data) from zip folders downloaded with 
+#' \code{\link{dataDWD}}.
+#' The metadata file \code{"Metadaten_Parameter.*txt"} in the zip folder \code{file} 
+#' is read, processed and returned as a data.frame.\cr
 #' \code{file} can be a vector with several filenames. 
 #' 
 #' @return data.frame of the desired dataset, 
 #'         or a named list of data.frames if length(file) > 1.
 #' @author Berry Boessenkool, \email{berry-b@@gmx.de}, Jun 2018
-#' @seealso \code{\link{dataDWD}}, \code{\link{readDWD}}, \code{\link{parameter_abbreviations}}
+#' @seealso \code{\link{dataDWD}}, \code{\link{readDWD}}, \code{\link{dwdparams}}
 #' @keywords file
 #' @importFrom utils read.table unzip
-#' @importFrom berryFunctions checkFile na9
+#' @importFrom berryFunctions checkFile na9 traceCall
 #' @importFrom pbapply pblapply
 #' @importFrom tools file_path_sans_ext
 #' @export
@@ -38,7 +40,6 @@ checkFile(file)
 output <- lapply(seq_along(file), function(i)
 {
 f <- file[i]
-
 # temporary unzipping directory
 fn <- tools::file_path_sans_ext(basename(f))
 exdir <- paste0(tempdir(),"/", fn)
@@ -46,39 +47,37 @@ unzip(f, exdir=exdir)
 on.exit(unlink(exdir, recursive=TRUE), add=TRUE)
 f <- dir(exdir, pattern="Metadaten_Parameter.*txt", full.names=TRUE)
 if(length(f)!=1) return(length(f))
-
 nr <- readLines(f) # number of rows
 nr <- sum(!substr(nr, 1, 7) %in% c("Legende", "generie"))
 tab <- read.table(f, na.strings=na9(), sep=";", header=TRUE, nrows=nr-1, 
                   stringsAsFactors=FALSE)
-
+#
 tab <- tab[,c("Parameter", "Parameterbeschreibung", "Einheit")]
 tab <- unique(tab)
-
+#
 dupli <- duplicated(tab$Parameter)
 if(any(dupli)) warning(traceCall(3, "", ": "), "The following entries are",
                        " duplicated: ", toString(unique(tab$Parameter[dupli])),
                        "\nThis occurs in '", fn, "/Metadaten_Parameter*.txt'",
                        call.=FALSE)
 rownames(tab) <- NULL
-
+#
 # Merge with short variable descriptions:
-tab2 <- merge(parameter_abbreviations, tab, all.y=TRUE)
+tab2 <- merge(dwdparams, tab, all.y=TRUE)
 kurzna <- is.na(tab2$Kurz)
 if(any(kurzna)) warning(traceCall(3, "", ": "), "The following entries are not",
                         " abbreviated yet: ", toString(tab2$Parameter[kurzna]),
                         "\nThis occurs in '", fn, "/Metadaten_Parameter*.txt'.",
                         "\nPlease inform berry-b@gmx.de so this can be included!\n",
                         call.=FALSE)
-
+#
 colnames(tab2)[1] <- "Par"
 rownames(tab2) <- tab2$Par
-
 # return column metadata:
 return(tab2)
 # lapply loop end
 })
-
+#
 # Warn about zip folders with no meta file:
 nometa <- sapply(output, class)=="integer"
 if(any(nometa)) 
@@ -88,14 +87,12 @@ if(any(nometa))
  mexp <- c("\nThis is expected since 1 and 10 minute data do not have ",
            "meta-information in most of the zip folders (as of 2019-02).\n")
  mnexp <- "\nPlease contact berry-b@gmx.de with with a copy of this warning.\n"
- 
  warning(traceCall(1, "", ": "), "The number of determined ",
          "'Metadaten_Parameter*.txt' files should be 1, but is instead:\n", 
          paste(msg[ exp],collapse="\n"), if(any( exp)) mexp, 
          paste(msg[!exp],collapse="\n"), if(any(!exp)) mnexp,
          call.=FALSE)
  }
-
 #
 names(output) <- tools::file_path_sans_ext(basename(file))
 output <- if(length(file)==1) output[[1]] else output
@@ -104,20 +101,26 @@ return(output)
 
 
 
+# dwdparams --------------------------------------------------------------------
 
-# parameter_abbreviations ---------------------------------------------------------------------
-
-#' Parameter abbreviations for data on the DWD CDC FTP server
+#' @title DWD parameter explanations
+#' @description Short German parameter explanations for the DWD abbreviations
+#' on the CDC FTP server.\cr
+#' These are manually created by me and might need to be expanded if the DWD adds
+#' more abbreviations.\cr
+#' \code{\link{readVars}} maps them to the variable abbreviations in the
+#' \code{"Metadaten_Parameter.*txt"} file in any given zip folder
+#' and will warn about missing entries.
 #' 
 #' @author Berry Boessenkool, \email{berry-b@@gmx.de}, Jun 2018
 #' @seealso \code{\link{readVars}}, \code{\link{readDWD}}
 #' @keywords datasets
 #' @export
 #' @examples
-#' head(parameter_abbreviations)
+#' head(dwdparams)
 #' 
-parameter_abbreviations <- unique(read.table(header=TRUE, strip.white=TRUE, 
-                                      stringsAsFactors=FALSE, text="
+dwdparams <- unique(read.table(header=TRUE, strip.white=TRUE,
+                               stringsAsFactors=FALSE, text="
 Parameter	Kurz
 JA_N	Bedeckungsgrad
 JA_N	Bedeckungsgrad
@@ -129,6 +132,8 @@ V_N	Bedeckungsgrad
 V_N	Bedeckungsgrad
 V_N	Bedeckungsgrad
 V_N	Bedeckungsgrad
+N_TER	Bedeckungsgrad
+N_TER	Bedeckungsgrad
 V_S1_NS	Bedeckungsgrad_Schicht1
 V_S1_NS	Bedeckungsgrad_Schicht1
 V_S2_NS	Bedeckungsgrad_Schicht2
@@ -139,6 +144,10 @@ V_S4_NS	Bedeckungsgrad_Schicht4
 V_S4_NS	Bedeckungsgrad_Schicht4
 VPM	Dampfdruck
 VPM	Dampfdruck
+VP_TER	Dampfdruck
+VP_TER	Dampfdruck
+E_TF_TER	Eisansatz
+E_TF_TER	Eisansatz
 V_TE002	Erdbodentemperatur_002cm
 V_TE002	Erdbodentemperatur_002cm
 V_TE002M	Erdbodentemperatur_002cm
@@ -161,6 +170,10 @@ V_TE050M	Erdbodentemperatur_050cm
 V_TE050M	Erdbodentemperatur_050cm
 V_TE100	Erdbodentemperatur_100cm
 V_TE100	Erdbodentemperatur_100cm
+EK_TER	Erdbodenzustand
+EK_TER	Erdbodenzustand
+TF_TER	Feuchttemperatur
+TF_TER	Feuchttemperatur
 PM	Luftdruck
 PM	Luftdruck
 PP_10	Luftdruck
@@ -179,30 +192,32 @@ TMK	Lufttemperatur
 TT_10	Lufttemperatur
 TT_TU	Lufttemperatur
 TT_TU	Lufttemperatur
+TT	Lufttemperatur
+TT	Lufttemperatur
 TM5_10	Lufttemperatur_5cm
 TX5_10	Lufttemperatur_5cm_max
 TGK	Lufttemperatur_5cm_min
 TGK	Lufttemperatur_5cm_min
 TN5_10	Lufttemperatur_5cm_min
-JA_MX_TX	Lufttemperatur_Max
-JA_MX_TX	Lufttemperatur_Max
+JA_MX_TX	Lufttemperatur_AbsMax
+JA_MX_TX	Lufttemperatur_AbsMax
+MX_TX	Lufttemperatur_AbsMax
+MX_TX	Lufttemperatur_AbsMax
+JA_MX_TN	Lufttemperatur_AbsMin
+JA_MX_TN	Lufttemperatur_AbsMin
+MX_TN	Lufttemperatur_AbsMin
+MX_TN	Lufttemperatur_AbsMin
 JA_TX	Lufttemperatur_Max
 JA_TX	Lufttemperatur_Max
 MO_TX	Lufttemperatur_Max
 MO_TX	Lufttemperatur_Max
-MX_TX	Lufttemperatur_Max
-MX_TX	Lufttemperatur_Max
-TX_10	Lufttemperatur_max
+TX_10	Lufttemperatur_Max
 TXK	Lufttemperatur_Max
 TXK	Lufttemperatur_Max
-JA_MX_TN	Lufttemperatur_Min
-JA_MX_TN	Lufttemperatur_Min
 JA_TN	Lufttemperatur_Min
 JA_TN	Lufttemperatur_Min
 MO_TN	Lufttemperatur_Min
 MO_TN	Lufttemperatur_Min
-MX_TN	Lufttemperatur_Min
-MX_TN	Lufttemperatur_Min
 TN_10	Lufttemperatur_Min
 TNK	Lufttemperatur_Min
 TNK	Lufttemperatur_Min
@@ -223,6 +238,7 @@ MO_RR	Niederschlagshoehe
 MO_RR	Niederschlagshoehe
 MO_RR	Niederschlagshoehe
 MO_RR	Niederschlagshoehe
+MO_RR	Niederschlagshoehe
 R1	Niederschlagshoehe
 R1	Niederschlagshoehe
 RS	Niederschlagshoehe
@@ -230,6 +246,7 @@ RS	Niederschlagshoehe
 RSK	Niederschlagshoehe
 RSK	Niederschlagshoehe
 RWS_10	Niederschlagshoehe
+RS_01	Niederschlagshoehe
 JA_MX_RS	Niederschlagshoehe_Max
 JA_MX_RS	Niederschlagshoehe_Max
 JA_MX_RS	Niederschlagshoehe_Max
@@ -238,9 +255,13 @@ MX_RS	Niederschlagshoehe_Max
 MX_RS	Niederschlagshoehe_Max
 MX_RS	Niederschlagshoehe_Max
 MX_RS	Niederschlagshoehe_Max
+MX_RS	Niederschlagshoehe_Max
+RTH_01	Niederschlagshoehe_Tropfen
+RWH_01	Niederschlagshoehe_Wippe
 RS_IND	Niederschlagsindikator
 RS_IND	Niederschlagsindikator
 RWS_IND_10	Niederschlagsindikator
+RS_IND_01	Niederschlagsindikator
 RF_10	Relative_Feuchte
 RF_TU	Relative_Feuchte
 RF_TU	Relative_Feuchte
@@ -248,10 +269,13 @@ UPM	Relative_Feuchte
 UPM	Relative_Feuchte
 RF_TER	Relative_Feuchte_Terminwert
 RF_TER	Relative_Feuchte_Terminwert
+RF_TER	Relative_Feuchte_Terminwert
+RF_TER	Relative_Feuchte_Terminwert
 JA_SH_S	Schneehoehe
 JA_SH_S	Schneehoehe
 MO_SH_S	Schneehoehe
 MO_SH_S	Schneehoehe
+MO_SH_S	Schneehoehe
 SH_TAG	Schneehoehe
 SH_TAG	Schneehoehe
 SH_TAG	Schneehoehe
@@ -264,12 +288,19 @@ JA_NSH	Schneehoehe_Neu
 JA_NSH	Schneehoehe_Neu
 MO_NSH	Schneehoehe_Neu
 MO_NSH	Schneehoehe_Neu
+MO_NSH	Schneehoehe_Neu
+NSH_TAG	Schneehoehe_Neu
+NSH_TAG	Schneehoehe_Neu
 WAAS_6	Schneewasseraequivalent
 WAAS_6	Schneewasseraequivalent
 WASH_6	Schneewasseraequivalent_Gesamt
 WASH_6	Schneewasseraequivalent_Gesamt
 V_VV	Sichtweite
 V_VV	Sichtweite
+V_VV	Sichtweite
+VK_TER	Sichtweite
+VK_TER	Sichtweite
+VK_TER	Sichtweite
 JA_SD_S	Sonnenscheindauer
 JA_SD_S	Sonnenscheindauer
 MO_SD_S	Sonnenscheindauer
@@ -291,28 +322,36 @@ FD_LBERG	Strahlung_Himmel_diffus
 FD_STRAHL	Strahlung_Himmel_diffus
 LS_10	Strahlung_langwellig
 TD_10	Taupunkttemperatur
+TD	Taupunkttemperatur
+TD	Taupunkttemperatur
 F	Windgeschwindigkeit
 F	Windgeschwindigkeit
 FF_10	Windgeschwindigkeit
 FM	Windgeschwindigkeit
 FM	Windgeschwindigkeit
-FMX_10	Windgeschwindigkeit_Max
+FF	Windgeschwindigkeit
+FF	Windgeschwindigkeit
 FX_10	Windgeschwindigkeit_Max
+FMX_10	Windgeschwindigkeit_MaxMean
 FNX_10	Windgeschwindigkeit_Min
 D	Windrichtung
 D	Windrichtung
 DD_10	Windrichtung
+DK_TER	Windrichtung
+DK_TER	Windrichtung
 DX_10	Windrichtung_Maxwind
-JA_MX_FX	Windspitze
-JA_MX_FX	Windspitze
 FX	Windspitze
 FX	Windspitze
+JA_MX_FX	Windspitze
+JA_MX_FX	Windspitze
 MX_FX	Windspitze
 MX_FX	Windspitze
 JA_FK	Windstaerke
 JA_FK	Windstaerke
 MO_FK	Windstaerke
 MO_FK	Windstaerke
+FK_TER	Windstaerke
+FK_TER	Windstaerke
 V_S1_CSA	Wolkenart_Abk_Schicht1
 V_S1_CSA	Wolkenart_Abk_Schicht1
 V_S2_CSA	Wolkenart_Abk_Schicht2
@@ -329,6 +368,8 @@ V_S3_CS	Wolkenart_Schicht3
 V_S3_CS	Wolkenart_Schicht3
 V_S4_CS	Wolkenart_Schicht4
 V_S4_CS	Wolkenart_Schicht4
+CD_TER	Wolkendichte
+CD_TER	Wolkendichte
 V_S1_HHS	Wolkenhoehe_Schicht1
 V_S1_HHS	Wolkenhoehe_Schicht1
 V_S2_HHS	Wolkenhoehe_Schicht2
@@ -338,3 +379,4 @@ V_S3_HHS	Wolkenhoehe_Schicht3
 V_S4_HHS	Wolkenhoehe_Schicht4
 V_S4_HHS	Wolkenhoehe_Schicht4
 "))
+rownames(dwdparams) <- dwdparams$Parameter
